/*
 * SimpleTopicLBClientServlet.java
 *
 */

package com.sun.appserver.ee.tests;

import java.io.*;
import java.net.*;

import javax.servlet.*;
import javax.servlet.http.*;

import javax.jms.*;
import javax.naming.*;
import java.util.*;
import javax.transaction.*;

import com.sun.jts.utils.RecoveryHooks.FailureInducer;
import java.sql.*;
import javax.ejb.EJB;
import javax.annotation.Resource;
import javax.sql.DataSource;

/**
 *
 * @author root
 * @version
 */
public class SimpleTopicLBClientServlet extends HttpServlet {
    
    private PrintWriter out = null;
    
    private javax.jms.TopicConnectionFactory quotesTCF;
    private javax.jms.Topic quotes;
    private javax.jms.TopicSession quotesSession = null;    
    private javax.jms.TopicConnection quotesConnection = null;    
    private javax.jms.TopicPublisher quotesPublisher = null;

    private javax.jms.MapMessage mapMessageRecd = null;
    private javax.jms.MapMessage mapMessageSent = null;
 
    private Context initialContext = null;
    private UserTransaction ut =null;

    final Integer NUM_MSGS =1; 
    private java.sql.Connection con = null;
    private java.sql.Statement stmt = null;
    private java.sql.Connection connection=null;
    private java.sql.PreparedStatement pstmt = null;
 
   /**
   * The init method looks up the QueueConectionFactory and Queue objects 
   * via JNDI service names and initializes variables 
   *
   * @exception ServletException
   */
    
    public void init() throws ServletException {   
        try{
            out("In init of servlet");
            // create the intial context
            initialContext = new InitialContext();

   
        }catch(Exception e){
            e.printStackTrace();
            throw new ServletException (e);
        }
    }
    
   /**
   * The initPublish method looks up the TopicConectionFactory and Topic objects 
   * via JNDI service names and initializes variables 
   *
   * @exception JMSException
   */    
    public void initPublish() throws JMSException{
        try{

            // look up the connection factory from the object store 
            out("Looking up the topic connection factory from JNDI");
            if (quotesTCF == null)
            quotesTCF = (javax.jms.TopicConnectionFactory) initialContext.lookup("java:comp/env/jms/QuotesCF");

            // look up topic from the object store
            out("Looking up the topic from JNDI");
            if (quotes == null)
            quotes = (javax.jms.Topic) initialContext.lookup("java:comp/env/jms/Quotes");
            

            // Creating a TopicConnection to the Message service");
            out("create a Topic Connection");
            quotesConnection = quotesTCF.createTopicConnection();
	
            // Starting the Connection
            out("start the connection");
            quotesConnection.start();

            // Creating a session within the connection
            out("create a session");
            quotesSession = quotesConnection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);

            // Creating a TopicPublisher 
            out("create a topicPublisher");
            quotesPublisher = quotesSession.createPublisher(quotes);
	
            // Building a message text
            out("create a map message");
            mapMessageSent = quotesSession.createMapMessage();
            String key = "key";
            for(int i = 0 ; i < NUM_MSGS ; i++){
                mapMessageSent.setString(key+i, "value"+i);                
            }
            
        }catch (NamingException e) {
            String msg = "An error was encountered trying to lookup an object from JNDI";
            out(msg);
            e.printStackTrace();
            throw new JMSException(e.getMessage());
        }catch (JMSException jmsex){
            String msg = "An error was encountered trying to create publisher/subscriber";
            out(msg);
            jmsex.printStackTrace();
            throw new JMSException(jmsex.getMessage());
        }catch(Exception ex){
            ex.printStackTrace();
            throw new JMSException(ex.getMessage());
        }
    } 
   
    private void createTable(){
        try{
            String createtableStatement = "Create Table MESSAGETABLE (data VARCHAR(100))";
            createConnection();
            stmt = con.createStatement();
            out("Creating the table");

            stmt.executeUpdate(createtableStatement);
        }catch(Exception ex){
            out("create table failed");
            ex.printStackTrace();
        }
    }
    private void dropTable(){
       try{
            createConnection();
            stmt = con.createStatement();
            out("Dropping the table");
            stmt.executeUpdate("drop table messagetable");

        }catch(Exception ex){
            ex.printStackTrace();
        }
    }
    private void createConnection(){
        try{
            if(con==null){
                Class.forName("org.apache.derby.jdbc.ClientDriver");
                con = DriverManager.getConnection("jdbc:derby://localhost:1527/testdb;user=dbuser;password=dbpassword;create=true;");
                stmt = con.createStatement();
                System.out.println("Done DB connection");
            }
        }catch(Exception ex){
            System.out.println("FAILED to create DB connection");
            ex.printStackTrace();
        }

    }

   private int readMessageFromDB() {
          Integer count = 0;
          StringBuffer buffer = new StringBuffer();
      try {
          String selectStatement = "select * from messagetable";
          pstmt = con.prepareStatement(selectStatement);
          ResultSet rs = pstmt.executeQuery();
          ResultSetMetaData rsmd = rs.getMetaData();
          int numCols = rsmd.getColumnCount();
          while(rs.next()){
             for(int i = 0 ; i < numCols; i++){
                buffer.append(String.valueOf(rs.getObject(i+1)));
                buffer.append(" ");
                count ++;
             }
          }
          pstmt.close();
          System.out.println("Got the following data " +buffer);
      }catch(Exception ex){
            System.out.println("Failed to read data from DB");
            ex.printStackTrace();
      }
      return count;
   }
 
    /** Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
     * @param request servlet request
     * @param response servlet response
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        response.setContentType("text/html;charset=UTF-8");
        out = response.getWriter();
        String action ="send";
        int count = 0;
        action = request.getParameter("action");
        log("OK Starting the processRequest");
        if("send".equalsIgnoreCase(action)){
            try{
                initPublish();
                out("begin send");
                out.println("begin send ");
                quotesPublisher.publish(mapMessageSent);
                out.println("<br>");
                out.println("end send ");
                out.println("<br>");
                out("end send");
            }catch(JMSException jmsex){
                String msg = "An error was encountered trying to send message";
                out(msg);
                jmsex.printStackTrace();
                throw new ServletException(msg, jmsex);            
            }catch(Exception ex){
                String msg = "An error was encountered trying to send message";
                out(msg);
                ex.printStackTrace();
                throw new ServletException(msg, ex);            
            }finally{
                destroyPublish();
            }
        }else {
            out.println("Correct way to invoke the servlet would be MessageServlet?action=send  <BR>");
        }        
        out.println("</body>");
        out.println("</html>");        
        out.close();
    }
    
    private void setHeaders(HttpServletRequest request){
        out.println("<html>");
        out.println("<head>");
        out.println("<title>Servlet SimpleTopicLBClientServlet</title>");
        out.println("</head>");
        out.println("<body>");
        out.println("<h1>Servlet SimpleTopicLBClientServlet at " + request.getContextPath () + "</h1>");
    }
    
    private void out(String message){
        System.out.println("Servlet " +message);
    }
    
    private void destroyPublish(){

//        if(quotesConnection != null){
            try{               
                out("Called destroy publish");
                quotesSession.close();
                quotesSession=null;
                quotesPublisher.close();
                quotesPublisher=null;
                quotesConnection.close();
                quotesConnection=null;
                out("Done destroy");
            }catch(Exception e){
                e.printStackTrace();
            }
//        }
        
    }
    
    public void destroy(){
        if(quotesConnection != null){
            try{
                quotesConnection.close();
                quotesConnection=null;
            }catch(Exception e){
                e.printStackTrace();
            }
        }
    }

    
    // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
    /** Handles the HTTP <code>GET</code> method.
     * @param request servlet request
     * @param response servlet response
     */
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        processRequest(request, response);
        /*try{
            initSubscribe();
        }catch(Exception ex){
            ex.printStackTrace();
        }*/
    }
    
    /** Handles the HTTP <code>POST</code> method.
     * @param request servlet request
     * @param response servlet response
     */
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        processRequest(request, response);
    }
    
    /** Returns a short description of the servlet.
     */
    public String getServletInfo() {
        return "Short description";
    }
    // </editor-fold>
}
